; Copyright (c) 2006-2007
; CACE Technologies
; All rights reserved.
; 
; Redistribution and use in source and binary forms, with or without 
; modification, are permitted provided that the following conditions 
; are met:
; 
; 1. Redistributions of source code must retain the above copyright 
; notice, this list of conditions and the following disclaimer.
; 2. Redistributions in binary form must reproduce the above copyright 
; notice, this list of conditions and the following disclaimer in the 
; documentation and/or other materials provided with the distribution. 
; 3. Neither the name of CACE nor the names of its 
; contributors may be used to endorse or promote products derived from 
; this software without specific prior written permission. 
; 
; THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
; "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
; LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR 
; A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
; OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
; SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
; LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, 
; DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY 
; THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
; (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
; OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
; 
;

;This includes the modern user interface
!include "MUI.nsh"

;--------------------------------
;Project definitions

  !define KISMET_PRJ_MAJOR "2007"
  !define KISMET_PRJ_MINOR "01"
  !define KISMET_PRJ_REV "R2"
  !define KISMET_PRJ_BUILD "0"
  !define KISMET_PROJ_VERSION_DOTTED "2006.11.1.0"
  !define KISMET_LEGAL_COPYRIGHT " 2006 Mike Kershaw"
  !define KISMET_PRODUCT_NAME "Kismet 2006-11-R1 for Windows"
  !define KISMET_COMPANY_NAME "CACE Technologies"
  !define KISMET_FILE_NAME "setup_kismet_${KISMET_PRJ_MAJOR}-${KISMET_PRJ_MINOR}-${KISMET_PRJ_REV}.exe"

; letter 'r'
  !define REINSTALL_FLAG "114"  
; letter u
  !define UNINSTALL_FLAG	"117"
; letter 'i'   
  !define INSTALL_FLAG "105"
 
  ;Default installation folder
  InstallDir "$PROGRAMFILES\Kismet"


;--------------------------------
;Includes and definitions


  ;the following two lines include the "Locate" function to find files
  !include "FileFunc.nsh"
  !insertmacro Locate

  !define VAR_0 0 ;$0
  !define VAR_1 1 ;$1
  !define VAR_2 2 ;$2
  !define VAR_3 3 ;$3
  !define VAR_4 4 ;$4
  !define VAR_5 5 ;$5
  !define VAR_6 6 ;$6
  !define VAR_7 7 ;$7
  !define VAR_8 8 ;$8
  !define VAR_9 9 ;$9

  ShowInstDetails nevershow
  ShowUninstDetails nevershow
  XPStyle on


  Var AIRPCAP_FOUND
  Var WINPCAP_FOUND

;--------------------------------
;General


;Name and file
  Name "${KISMET_PRODUCT_NAME}"
  OutFile "${KISMET_FILE_NAME}"
  
;Get Installation folder from registry if available
  InstallDirRegKey HKLM "Software\Kismet" ""
  
; Icon of installer and uninstaller
;  Icon "..\installation_cd_files\autorun\autorun.ico"
;  UninstallIcon "modern-uninstall.ico"

;--------------------------------
;User interface Configuration

  !define MUI_HEADERIMAGE
  !define MUI_HEADERIMAGE_BITMAP "kismet.bmp"
  !define MUI_ABORTWARNING
  !define MUI_CUSTOMFUNCTION_GUIINIT myGuiInit
  !define MUI_CUSTOMFUNCTION_ABORT .onInstFailed
  !define MUI_WELCOMEPAGE_TEXT "This Wizard will guide you through the entire Kismet software installation.$\nFor more information or support, please visit the CACE Technologies web site:$\n$\nhttp://www.cacetech.com"
;  !define MUI_ICON "..\installation_cd_files\autorun\autorun.ico"
;  !define MUI_UNICON "modern-uninstall.ico"

; This is needed to give focus to the main window after the initial "installer loading..." dialog is shown.
  Function myGUIInit
    ShowWindow $HWNDPARENT 2
  FunctionEnd

;--------------------------------
;Installer Pages

  ;Installer
  
  !insertmacro MUI_PAGE_WELCOME
  !insertmacro MUI_PAGE_LICENSE "license.txt"
  !insertmacro MUI_PAGE_INSTFILES
  !insertmacro MUI_PAGE_FINISH

  ;Uninstaller
  !insertmacro MUI_UNPAGE_CONFIRM
  !insertmacro MUI_UNPAGE_INSTFILES
  !insertmacro MUI_UNPAGE_FINISH
  
;--------------------------------
;Languages
!insertmacro MUI_LANGUAGE "English"

;--------------------------------
;Init function: copy the files we need during the installer

  Function .onInit

; Initialize the plugin directorty. Needed??
    InitPluginsDir

; Deny any attempt to start a silent installation
    SetSilent normal

;Check if winpcap and airpcap are installed
;after this call, all the AIRPCAP_OLD_xxx variables are correctly set
  Call IsWinPcapInstalled
  Call IsAirPcapInstalled

  StrCmp $AIRPCAP_FOUND "false" AirPcapNotAlreadyPresent
  StrCmp $WINPCAP_FOUND "false" WinPcapNotAlreadyPresent
  goto WinPcapCheckDone

AirPcapNotAlreadyPresent:
WinPcapNotAlreadyPresent:
    MessageBox MB_ICONEXCLAMATION|MB_OK "BEFORE INSTALLING KISMET ON WINDOWS, YOU NEED TO INSTALL WINPCAP AND AIRPCAP.$\n$\nWinPcap can be downloaded from http://www.winpcap.org/install/default.htm$\nAirPcap can be downloaded from http://www.cacetech.com/support/downloads.htm$\n$\nTHE INSTALLATION WILL NOW ABORT."
    Quit   

WinPcapCheckDone:

  FunctionEnd


;--------------------------------
;Init function for the uninstaller: disable support for silent removal
  Function un.onInit
    InitPluginsDir
; Deny any attempt to start a silent uninstallation of Kismet
    SetSilent normal

  FunctionEnd

;--------------------------------
;Called when the installation fails. Just remove any temp file on the machine.
  Function .onInstFailed
    call CleanupTempFiles
  FunctionEnd

;--------------------------------
;Called when the installation succeeds. Just remove any temp file on the machine.
  Function .onInstSuccess
    call CleanupTempFiles
  FunctionEnd

;--------------------------------
;Remove any temp file used during the installation
  Function CleanupTempFiles
  FunctionEnd

;--------------------------------
;Main Installer Section
Section "Main Installer Section" MainInstall

;
; First of all, we delete the intermediate files used only by the installer
;
    Call CleanupTempFiles

;Assume no reboot by default
    SetRebootFlag false


;
;tell the installer to put the start menu stuff under "all users"
;this prevents NSIS from having craziness during uninstallation
;
; The best should be to have it on the admin (=current) user, but
; the link to Kismet is useful, as well.
    SetShellVarContext all

;
;Copy Files
;

;Files in program files

    SetOutPath "$INSTDIR"

    File "Files\cygncurses-8.dll"
    File "Files\cygpanel-8.dll"
    File "Files\cygwin1.dll"
    File "Files\kismet.conf"
    File "Files\KismetRunner.exe"
    File "Files\kismet_client.exe"
    File "Files\kismet_drone.conf"
    File "Files\kismet_drone.exe"
    File "Files\kismet_server.exe"
    File "Files\kismet_ui.conf"
    File "Files\kismet_drone.exe"
    File "Files\kismet_drone.exe"
    File "Files\kismet_drone.exe"
    File "Files\KismetConf.exe"
    File "Files\Notepad2.exe"
    File "Files\kill.exe"
    CreateDirectory "$INSTDIR\logs"
  
    CreateDirectory "$PROFILE\.terminfo"
    CreateDirectory "$PROFILE\.terminfo\c"
    SetOutPath "$PROFILE\.terminfo\c"

    File "Files\.terminfo\c\cygwin"
    File "Files\.terminfo\c\cygwinB19"
    File "Files\.terminfo\c\cygwinDBG"

EndCopy:

;
;Store installation folder
;
    WriteRegStr HKLM "SOFTWARE\Kismet" "" $INSTDIR
 
;
;Create an "add/remove programs" entry
;
    WriteRegStr HKLM \
	"Software\Microsoft\Windows\CurrentVersion\Uninstall\kismetinst" \ 
	"DisplayName" "${KISMET_PRODUCT_NAME}"

    WriteRegStr HKLM \
	"SOFTWARE\Microsoft\Windows\CurrentVersion\Uninstall\kismetinst" \ 
	"UninstallString" "$INSTDIR\uninstall.exe"  

    WriteRegStr HKLM \
	"SOFTWARE\Microsoft\Windows\CurrentVersion\Uninstall\kismetinst" \ 
	"Publisher" "CACE Technologies"  

    WriteRegStr HKLM \
	"SOFTWARE\Microsoft\Windows\CurrentVersion\Uninstall\kismetinst" \ 
	"URLInfoAbout" "http://www.cacetech.com"
  
    WriteRegStr HKLM \
	"SOFTWARE\Microsoft\Windows\CurrentVersion\Uninstall\kismetinst" \ 
	"URLUpdateInfo" "http://www.cacetech.com"

    WriteRegStr HKLM \
	"SOFTWARE\Microsoft\Windows\CurrentVersion\Uninstall\kismetinst" \ 
	"VersionMajor" "${KISMET_PRJ_MAJOR}"  

    WriteRegStr HKLM \
	"SOFTWARE\Microsoft\Windows\CurrentVersion\Uninstall\kismetinst" \ 
	"VersionMinor" "${KISMET_PRJ_MINOR}"  

    WriteRegStr HKLM \
	"SOFTWARE\Microsoft\Windows\CurrentVersion\Uninstall\kismetinst" \ 
	"DisplayVersion" "${KISMET_PROJ_VERSION_DOTTED}"

AddKismetIcon:

    WriteRegStr HKLM \
	"SOFTWARE\Microsoft\Windows\CurrentVersion\Uninstall\kismetinst" \ 
	"DisplayIcon" "$INSTDIR\KismetRunner.exe"

NoKismetIcon:

    CreateDirectory "$SMPROGRAMS\Kismet"

;
;Create start menu shortcuts
;
    SetOutPath "$INSTDIR"
    CreateShortCut "$SMPROGRAMS\Kismet\Kismet.lnk" "$INSTDIR\KismetRunner.exe" "" "$INSTDIR\KismetRunner.exe" 0 "" "" "Kismet"
    CreateShortCut "$SMPROGRAMS\Kismet\Kismet Configuration Console.lnk" "$INSTDIR\KismetConf.exe" "" "$INSTDIR\KismetConf.exe" 0 "" "" "Kismet Configuration Console"
    CreateShortCut "$SMPROGRAMS\Kismet\Command Line Prompt.lnk" "$SYSDIR\cmd.exe" "" "$SYSDIR\cmd.exe" 0 "" "" "Open a DOS Console in the Kismet Folder"
	CreateShortCut "$SMPROGRAMS\Kismet\Kismet Logs Folder.lnk" "$INSTDIR\logs"

;Create uninstaller
    WriteUninstaller "$INSTDIR\Uninstall.exe"

EndInstallation:

SectionEnd

;--------------------------------
;Language-related stuff
;Language strings
  LangString DESC_MainInstall ${LANG_ENGLISH} "Kismet installation section. No panels"

  ;Assign language strings to sections
  !insertmacro MUI_FUNCTION_DESCRIPTION_BEGIN
  !insertmacro MUI_DESCRIPTION_TEXT ${MainInstall} $(DESC_MainInstall)
  !insertmacro MUI_FUNCTION_DESCRIPTION_END
 
;--------------------------------
;Uninstaller Section
Section "Uninstall" MainUnistall

    SetRebootFlag false

    SetShellVarContext all

;Delete the files

    SetOutPath "$INSTDIR"

    Delete "cygncurses-8.dll"
    Delete "cygpanel-8.dll"
    Delete "cygwin1.dll"
    Delete "kismet.conf"
    Delete "KismetRunner.exe"
    Delete "kismet_client.exe"
    Delete "kismet_drone.conf"
    Delete "kismet_drone.exe"
    Delete "kismet_server.exe"
    Delete "kismet_ui.conf"
    Delete "kismet_drone.exe"
    Delete "kismet_drone.exe"
  
    SetOutPath "$PROFILE"

    Delete ".terminfo\c\cygwin"
    Delete ".terminfo\c\cygwinB19"
    Delete ".terminfo\c\cygwinDBG"
    RMDir /r "$PROFILE\.terminfo"

;
;remove start menu entries
;

    SetOutPath "$SMPROGRAMS"
    RMDir /r "$SMPROGRAMS\Kismet"

;
;delete the installation directory
;
    RMDir /r "$INSTDIR"

;
;remove registry entries
;
    DeleteRegKey /ifempty HKLM "Software\Kismet"
    DeleteRegKey HKLM "SOFTWARE\Microsoft\Windows\CurrentVersion\Uninstall\kismetinst"

SectionEnd

;--------------------------------
;Installer Version Information
  VIProductVersion "${KISMET_PROJ_VERSION_DOTTED}"
  VIAddVersionKey /LANG=${LANG_ENGLISH} "ProductName" "${KISMET_PRODUCT_NAME}"
  VIAddVersionKey /LANG=${LANG_ENGLISH} "Comments" ""
  VIAddVersionKey /LANG=${LANG_ENGLISH} "CompanyName" "${KISMET_COMPANY_NAME}"
  VIAddVersionKey /LANG=${LANG_ENGLISH} "LegalTrademarks" ""
  VIAddVersionKey /LANG=${LANG_ENGLISH} "LegalCopyright" "${KISMET_LEGAL_COPYRIGHT}"
  VIAddVersionKey /LANG=${LANG_ENGLISH} "FileDescription" "${KISMET_PRODUCT_NAME} installer"
  VIAddVersionKey /LANG=${LANG_ENGLISH} "FileVersion" "${KISMET_PROJ_VERSION_DOTTED}"

;--------------------------------
;this function checks if AirPcap is installed, and stores all the information into the AIRPCAP_OLD_xxx variable
Function IsAirPcapInstalled

  StrCpy $AIRPCAP_FOUND "false"
  ${Locate} "$SYSDIR" "/M=airpcap.dll" 'AirpcapFound'

FunctionEnd

; this callback is called when packet.dll is found. It stores the version of the dll 
; into  AIRPCAP_OLD_PROJ_VERSION_DOTTED
Function AirpcapFound
  StrCpy $AIRPCAP_FOUND "true"
FunctionEnd

;--------------------------------
;this function checks if WinPcap is installed, and stores all the information into the WINPCAP_OLD_xxx variable
Function IsWinPcapInstalled

  StrCpy $WINPCAP_FOUND "false"
  ${Locate} "$SYSDIR" "/M=packet.dll" 'PacketFound'

FunctionEnd

; this callback is called when packet.dll is found. It stores the version of the dll 
; into  WINPCAP_OLD_PROJ_VERSION_DOTTED
Function PacketFound
  StrCpy $WINPCAP_FOUND "true"
FunctionEnd

