/*
 * Copyright (c) 2006-2007 CACE Technologies, Davis (California)
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 * 3. Neither the name of CACE Technologies nor the names of its 
 * contributors may be used to endorse or promote products derived 
 * from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */

#include "stdafx.h"
#include "airpcap.h"
#include "KismetConf.h"
#include "KismetConfDlg.h"
#include "DlgDroneDetails.h"
#include "ConfigFile.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CKismetConfDlg dialog

CKismetConfDlg::CKismetConfDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CKismetConfDlg::IDD, pParent)
{
	//{{AFX_DATA_INIT(CKismetConfDlg)
	//}}AFX_DATA_INIT
	// Note that LoadIcon does not require a subsequent DestroyIcon in Win32
	m_hIcon = AfxGetApp()->LoadIcon(IDR_MAINFRAME);
	
	m_AdaptersList.RemoveAll();
}

void CKismetConfDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CKismetConfDlg)
	DDX_Control(pDX, IDC_LIST_DRONES, m_ListDrones);
	DDX_Control(pDX, IDC_COMBO_SOURCES, m_ComboSources);
	DDX_Control(pDX, IDC_CHECK_ENABLE_LOGGING, m_CheckEnableLogging);
	DDX_Control(pDX, IDC_BUTTON_EDIT_UI, m_ButtonEditUiConf);
	DDX_Control(pDX, IDC_BUTTON_EDIT_DRONE, m_ButtonEditDroneConf);
	DDX_Control(pDX, IDC_BUTTON_EDIT_CONF, m_ButtonEditKismetConf);
	DDX_Control(pDX, IDC_BUTTON_DRONES_REMOVE, m_ButtonDronesRemove);
	DDX_Control(pDX, IDC_BUTTON_DRONES_CLEAR, m_ButtonDronesClear);
	DDX_Control(pDX, IDC_BUTTON_DRONES_ADD, m_ButtonDronesAdd);
	DDX_Control(pDX, ID_BUTTON_RUNKISMET, m_ButtonRunKismet);
	//}}AFX_DATA_MAP
}

BEGIN_MESSAGE_MAP(CKismetConfDlg, CDialog)
	//{{AFX_MSG_MAP(CKismetConfDlg)
	ON_WM_PAINT()
	ON_WM_QUERYDRAGICON()
	ON_BN_CLICKED(IDC_RADIO_SELECT, OnRadioSpecifyAdapter)
	ON_BN_CLICKED(IDC_RADIO_NOLOCAL, OnRadioNoLocalSources)
	ON_BN_CLICKED(IDC_RADIO_ALLLOCAL, OnRadioAllLocalSources)
	ON_BN_CLICKED(IDC_BUTTON_EDIT_CONF, OnButtonEditConf)
	ON_BN_CLICKED(IDC_BUTTON_EDIT_UI, OnButtonEditUi)
	ON_BN_CLICKED(IDC_BUTTON_EDIT_DRONE, OnButtonEditDrone)
	ON_BN_CLICKED(IDC_BUTTON_DRONES_ADD, OnButtonDronesAdd)
	ON_BN_CLICKED(IDC_BUTTON_DRONES_CLEAR, OnButtonDronesClear)
	ON_BN_CLICKED(IDC_BUTTON_DRONES_REMOVE, OnButtonDronesRemove)
	ON_BN_CLICKED(ID_BUTTON_RUNKISMET, OnButtonRunkismet)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CKismetConfDlg message handlers

BOOL CKismetConfDlg::OnInitDialog()
{
	CDialog::OnInitDialog();

	// Set the icon for this dialog.  The framework does this automatically
	//  when the application's main window is not a dialog
	SetIcon(m_hIcon, TRUE);			// Set big icon
	SetIcon(m_hIcon, FALSE);		// Set small icon
	
	//
	// Init Controls
	//
	m_ListDrones.SetExtendedStyle(m_ListDrones.GetExtendedStyle() | LVS_EX_GRIDLINES | LVS_EX_FULLROWSELECT);
    
	CRect l_rect;
    m_ListDrones.GetClientRect(&l_rect);
	m_ListDrones.InsertColumn(0, "Not Shown", LVCFMT_LEFT, l_rect.Size().cx);

	FillAdapterList();
	m_ComboSources.EnableWindow(FALSE);
	m_CheckEnableLogging;
	((CButton*)GetDlgItem(IDC_RADIO_ALLLOCAL))->SetCheck(TRUE);

	//
	// Load the configuration from file
	//
	LoadConfig();
	
	return TRUE;  // return TRUE  unless you set the focus to a control
}

// If you add a minimize button to your dialog, you will need the code below
//  to draw the icon.  For MFC applications using the document/view model,
//  this is automatically done for you by the framework.

void CKismetConfDlg::OnPaint() 
{
	if (IsIconic())
	{
		CPaintDC dc(this); // device context for painting

		SendMessage(WM_ICONERASEBKGND, (WPARAM) dc.GetSafeHdc(), 0);

		// Center icon in client rectangle
		int cxIcon = GetSystemMetrics(SM_CXICON);
		int cyIcon = GetSystemMetrics(SM_CYICON);
		CRect rect;
		GetClientRect(&rect);
		int x = (rect.Width() - cxIcon + 1) / 2;
		int y = (rect.Height() - cyIcon + 1) / 2;

		// Draw the icon
		dc.DrawIcon(x, y, m_hIcon);
	}
	else
	{
		CDialog::OnPaint();
	}
}

// The system calls this to obtain the cursor to display while the user drags
//  the minimized window.
HCURSOR CKismetConfDlg::OnQueryDragIcon()
{
	return (HCURSOR) m_hIcon;
}

void CKismetConfDlg::OnRadioSpecifyAdapter() 
{
	m_ComboSources.EnableWindow(TRUE);
}

void CKismetConfDlg::OnRadioNoLocalSources() 
{
	m_ComboSources.EnableWindow(FALSE);
}

void CKismetConfDlg::OnRadioAllLocalSources() 
{
	m_ComboSources.EnableWindow(FALSE);
}

/////////////////////////////////////////////////////////////////////////////
// CKismetConfDlg Methods

//
// Run a command asynchronously inside the current shell
//
HANDLE CKismetConfDlg::CmdExec(char * Cmd, char* args, BOOL Wait)
{
	BOOL Res = FALSE;
	STARTUPINFO si;
	PROCESS_INFORMATION pi;
	char commandstring[256];
	DWORD Flags = 0;
	
	// 
	// Configure the input parameters
	//
	memset(&si, 0, sizeof(STARTUPINFO));
	si.cb = sizeof(STARTUPINFO);
	si.wShowWindow = SW_SHOWMINIMIZED;

	// 
	// Execute the command with a call to the CreateProcess API call.
	//
	
	sprintf(commandstring, "%s %s", Cmd, args);
	
	Res = CreateProcess(NULL,
		commandstring,
		NULL,
		NULL,
		TRUE,
		Flags,
		NULL,
		NULL,
		&si,
		&pi);
	
	if(!Res)
	{
		return NULL;
	}
	

	if(Wait)
	{
		WaitForSingleObject(pi.hProcess, INFINITE);
	}
	
	return pi.hProcess;
}

UINT CKismetConfDlg::FillAdapterList()
{
	UINT i;
	UINT Nadapts;
	AirpcapDeviceDescription *DevsList, *AdListEntry;
	CHAR Ebuf[AIRPCAP_ERRBUF_SIZE];
	AdapterIdentifier tIdentifier;

	m_AdaptersList.RemoveAll();

	//
	// Clear the combobox
	//
	m_ComboSources.ResetContent();

	//
	// Get the list of adapters
	//
	if(!AirpcapGetDeviceList(&DevsList, Ebuf))
	{
		// Error getting the interfaces list, return
		return KC_FAILURE;
	}

	//
	// Count the adapters
	//
	AdListEntry = DevsList;
	Nadapts = 0;
	while(AdListEntry)
	{
		Nadapts++;
		AdListEntry = AdListEntry->next;
	}

	if(Nadapts == 0)
	{
		// No interfaces, return
		AirpcapFreeDeviceList(DevsList);
		return KC_FAILURE;
	}

	//
	// Insert the adapters in our list
	//
	AdListEntry = DevsList;
	for(i = 0; i < Nadapts; i++)
	{
		//
		// NOTE: we put them in the reverse order because AirpcapGetDevices gives us the last first
		//
		tIdentifier.Name = AdListEntry->Name;
		tIdentifier.Description = AdListEntry->Description;
		m_AdaptersList.Add(tIdentifier);

		AdListEntry = AdListEntry->next;
	}

	AirpcapFreeDeviceList(DevsList);

	for(i = 0; i < m_AdaptersList.GetSize(); i++)
	{
		m_ComboSources.InsertString(-1, m_AdaptersList[i].Name);
	}
	m_ComboSources.SetCurSel(0);

	return KC_SUCCESS;
}

void CKismetConfDlg::OnButtonEditConf() 
{
	CKismetConfDlg::CmdExec("notepad2.exe", "kismet.conf", FALSE);
}

void CKismetConfDlg::OnButtonEditUi() 
{
	CKismetConfDlg::CmdExec("notepad2.exe", "kismet_ui.conf", FALSE);
}

void CKismetConfDlg::OnButtonEditDrone() 
{
	CKismetConfDlg::CmdExec("notepad2.exe", "kismet_drone.conf", FALSE);
}

void CKismetConfDlg::OnButtonDronesAdd() 
{	
	CDlgDroneDetails Dlg;

	Dlg.m_SourceString = "";
	
	if(Dlg.DoModal() == IDOK)
	{
		int ItemCount = m_ListDrones.GetItemCount();
		m_ListDrones.InsertItem(ItemCount, Dlg.m_SourceString);
	}
}

void CKismetConfDlg::OnButtonDronesRemove() 
{
	UINT SelItem = m_ListDrones.GetNextItem(-1, LVNI_SELECTED);
	m_ListDrones.DeleteItem(SelItem);

	if(m_ListDrones.GetItemCount() > 1)
	{
		m_ListDrones.SetItemState(SelItem, TVIS_SELECTED,  TVIS_SELECTED);
	}
	else
	{
		m_ListDrones.SetItemState(0, TVIS_SELECTED,  TVIS_SELECTED);
	}
}

void CKismetConfDlg::OnButtonDronesClear() 
{
	m_ListDrones.DeleteAllItems();
	m_AdaptersList.RemoveAll();
}

UINT CKismetConfDlg::SaveConfig() 
{
	CFile f;
	CString s, s1;
	INT i;

	// make sure that if the "select source" is checked, at least
	// one adapter is present
	if(((CButton*)GetDlgItem(IDC_RADIO_SELECT))->GetCheck())
	{
		if(m_ComboSources.GetCurSel() == -1)
		{
			MessageBox("When 'Specify Adapter' is selected as a local source, at least one AirPcap adapter must be present");
			return KC_CANNOTCLOSE;
		}
	}

	// create the file
	if(!f.Open("KismetConf.cfg", CFile::modeReadWrite | CFile::modeCreate))
	{
		MessageBox("Unable to open the configuration file for writing");
		return KC_FAILURE;
	}

	s.Format("; Generated by KismetConf.exe\n; Do not edit\n");
	f.Write(s, s.GetLength());

	s.Format("[KismetConf]\n");
	f.Write(s, s.GetLength());

	// save source type
	if(((CButton*)GetDlgItem(IDC_RADIO_NOLOCAL))->GetCheck())
	{
		s.Format("LocalSourceType = none\n");
		f.Write(s, s.GetLength());
	}
	else
	if(((CButton*)GetDlgItem(IDC_RADIO_ALLLOCAL))->GetCheck())
	{
		s.Format("LocalSourceType = all\n");
		f.Write(s, s.GetLength());
	}
	else
	if(((CButton*)GetDlgItem(IDC_RADIO_SELECT))->GetCheck())
	{
		s.Format("LocalSourceType = specify\n");
		f.Write(s, s.GetLength());

		m_ComboSources.GetLBText(m_ComboSources.GetCurSel(), s1);
		s.Format("LocalSource = %s\n", s1);
		f.Write(s, s.GetLength());
	}

	// save drones list
	for(i = 0; i < m_ListDrones.GetItemCount(); i++)
	{
		s1 = m_ListDrones.GetItemText(i, 0);
		s.Format("Drone%u = %s\n", i, s1);
		f.Write(s, s.GetLength());
	}

	// save logging state
	if(m_CheckEnableLogging.GetCheck())
	{
		s.Format("EnableLogging = yes\n");
	}
	else
	{
		s.Format("EnableLogging = no\n");
	}

	f.Write(s, s.GetLength());

	f.Close();

	return KC_SUCCESS;
}

UINT CKismetConfDlg::LoadConfig() 
{
	ConfigFile cf("KismetConf.cfg");
	CString s, s1;
	UINT i;

	// source type
	s = cf.Value("KismetConf","LocalSourceType").c_str();
	if(s == "")
	{
		goto DoneLoading;
	}
	s.MakeLower();

	if(s == "none")
	{
		((CButton*)GetDlgItem(IDC_RADIO_NOLOCAL))->SetCheck(TRUE);
		((CButton*)GetDlgItem(IDC_RADIO_ALLLOCAL))->SetCheck(FALSE);
		((CButton*)GetDlgItem(IDC_RADIO_SELECT))->SetCheck(FALSE);
	}
	else
	if(s == "all")
	{
		((CButton*)GetDlgItem(IDC_RADIO_NOLOCAL))->SetCheck(FALSE);
		((CButton*)GetDlgItem(IDC_RADIO_ALLLOCAL))->SetCheck(TRUE);
		((CButton*)GetDlgItem(IDC_RADIO_SELECT))->SetCheck(FALSE);
	}
	else
	if(s == "specify")
	{
		((CButton*)GetDlgItem(IDC_RADIO_NOLOCAL))->SetCheck(FALSE);
		((CButton*)GetDlgItem(IDC_RADIO_ALLLOCAL))->SetCheck(FALSE);
		((CButton*)GetDlgItem(IDC_RADIO_SELECT))->SetCheck(TRUE);
		m_ComboSources.EnableWindow(TRUE);

		s = cf.Value("KismetConf","LocalSource").c_str();
		s.MakeLower();
		m_ComboSources.SelectString(0, s);
	}

	// drones list
	for(i = 0; i < 100; i++)
	{
		s1.Format("Drone%u", i);

		s = cf.Value("KismetConf", s1.GetBuffer(1)).c_str();
		if(s == "")
		{
			break;
		}

		int ItemCount = m_ListDrones.GetItemCount();
		m_ListDrones.InsertItem(ItemCount, s);
	}

	// logging state
	s = cf.Value("KismetConf","EnableLogging").c_str();
	s.MakeLower();
	if(s == "")
	{
		goto DoneLoading;
	}
	else
	if(s == "yes")
	{
		m_CheckEnableLogging.SetCheck(TRUE);
	}
	else
	if(s == "no")
	{
		m_CheckEnableLogging.SetCheck(FALSE);
	}

DoneLoading:

	return KC_SUCCESS;
}

void CKismetConfDlg::OnOK() 
{
	if(SaveConfig() != KC_CANNOTCLOSE)
	{
		CDialog::OnOK();
	}
}

void CKismetConfDlg::OnButtonRunkismet() 
{
	SaveConfig();
	CKismetConfDlg::CmdExec("KismetRunner.exe", "", FALSE);	
}
